# -*- coding: utf-8 -*-
"""
/***************************************************************************
 walidatorPlikowGML
                                 A QGIS plugin
 Walidator plików GML baz BDOT10k, PRNG, GESUT, EGiB, BDOT500
 Generated by Plugin Builder: http://g-sherman.github.io/Qgis-Plugin-Builder/
                              -------------------
        begin                : 2022-12-23
        git sha              : $Format:%H$
        copyright            : (C) 2022 by Marcin Lebiecki - Główny Urząd Geodezji i Kartografii
        email                : marcin.lebiecki@gugik.gov.pl
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
"""
from qgis.core import *
from qgis import processing
import pathlib
import configparser
import sys


def findDuplicates(layer):
    duplicates = []
    unique = []
    isFirstUnique = True
    for feature in layer.getFeatures():
        isDuplicate = False
        if isFirstUnique:
            unique.append(feature)
            isFirstUnique = False
        else:
            for obj in unique:
                if obj.geometry().isGeosEqual(feature.geometry()):
                    isDuplicate = True
                    break
            if isDuplicate:
                duplicates.append(obj)
            else:
                unique.append(feature)
    return duplicates


def validateGeometry(layer):
    obiektyZbledami = []
    wynik = processing.run("qgis:checkvalidity", {
        'INPUT_LAYER': layer,
        'METHOD': 1,
        'INVALID_OUTPUT': 'TEMPORARY_OUTPUT'
    })
    layer = wynik['INVALID_OUTPUT']
    
    for obj in layer.getFeatures():
        obiektyZbledami.append(obj)
    return obiektyZbledami


def granicePowiatow():
    config = configparser.ConfigParser()
    mainPath = pathlib.Path(QgsApplication.qgisSettingsDirPath())/pathlib.Path(r"python/plugins/Walidator_plikow_gml/")
    config.read(str(mainPath)+'/Walidator_plikow_gml.ini')
    granicePowiatowPath = config['sciezkaPowiaty']['shpPowiaty']
    granicePowiatow_A = QgsVectorLayer(granicePowiatowPath, 'GranicePowiatow', 'ogr')
    
    # rozbicie multipoligon na poligony
    pojedynczeGranice = processing.run("native:multiparttosingleparts", {
        'INPUT': granicePowiatow_A,
        'OUTPUT': 'memory:'
    })
    
    # zamiana typu geometrii z poligonu na linię
    granicePowiatow_L = processing.run("native:polygonstolines", {
        'INPUT': pojedynczeGranice['OUTPUT'],
        'OUTPUT': 'memory:'
    })
    return granicePowiatow_L


def czyPrzecinaGranicePowiatuDlugoscPonizej50m(layer):
    try:
        obiektyZbledami = []
        granicePowiatow_L = granicePowiatow()
        # Dodaj wynikową warstwę liniową do projektu
        alg_params = {
            'INPUT': granicePowiatow_L['OUTPUT'],
            'DISTANCE': 7.5,  # Odległość bufora
            'SEGMENTS': 8,  # Ilość segmentów na kwadrant, większa wartość = bardziej okrągły bufor
            'END_CAP_STYLE': 0,  
            'JOIN_STYLE': 0,
            'MITER_LIMIT': 2,
            'DISSOLVE': False,  # Ustawienie na True spowoduje scalenie wszystkich buforów w jeden obiekt
            'OUTPUT': 'memory:'
        }
        # Tworzenie bufora
        buforGranicy = processing.run('qgis:buffer', alg_params)['OUTPUT']
        QgsProject.instance().addMapLayer(buforGranicy)
    
        for obj in layer.getFeatures():
            if obj.geometry().length() < 50:
                czyPrzecina = False
                for granica in buforGranicy.getFeatures():
                    if obj.geometry().intersects(granica.geometry()): 
                        czyPrzecina = True
                if czyPrzecina:
                    obiektyZbledami.append(obj)          
        return obiektyZbledami
    except:
        QMessageBox.critical(QMessageBox(),'Brak warstwy referencyjnej','Sprawdź ścieżkę do pliku z granicami powiatów w Walidator_plikow_gml.ini', QMessageBox.Ok)


def czyObiektyWewnatrzPowiatu(layer, teryt):
    obiektyZbledami = []
    config = configparser.ConfigParser()
    mainPath = pathlib.Path(QgsApplication.qgisSettingsDirPath())/pathlib.Path("python/plugins/Walidator_plikow_gml/")
    config.read(str(mainPath)+'/Walidator_plikow_gml.ini')
    granicePowiatowPath = config['sciezkaPowiaty']['shpPowiaty']
    granicePowiatow_A = QgsVectorLayer(granicePowiatowPath, 'GranicePowiatow', 'ogr')
    request = QgsFeatureRequest(QgsExpression("jpt_kod_je = \'" + teryt + "\'"))
    requestFeatures = granicePowiatow_A.getFeatures(request)
    for requestFeature in requestFeatures:
        granicaPowiatu = requestFeature
        break
    if granicaPowiatu is not None:
        geom = granicaPowiatu.geometry()
        granica = geom.convertToType(QgsWkbTypes.LineGeometry) # konwersja poligona na linię, w celu wydobycia granicy
        
    for obj in layer.getFeatures():
        objGeom = obj.geometry()
        vertices = list(objGeom.vertices())
        numVertices = len(vertices)  # Liczba wierzchołków
        p1 = QgsPointXY(vertices[0])
        p2 = QgsPointXY(vertices[-1])
        # Konwersja na QgsGeometry
        geomP1 = QgsGeometry.fromPointXY(p1)
        geomP2 = QgsGeometry.fromPointXY(p2)
        # Konwersja QgsPointXY na QgsGeometry przed użyciem distance()
        d1 = granica.distance(geomP1)
        d2 = granica.distance(geomP2)
        if d1 > 0.01 and d2 > 0.01:
            if not geom.contains(obj.geometry()):
                obiektyZbledami.append(obj)
    return obiektyZbledami


def czyOdleglosciMiedzyPoziomicami2m_wersja1(layer):
    obiektyZbledami = []
    spatial_index = QgsSpatialIndex(layer.getFeatures())
    
    for obj1 in layer.getFeatures():
        for obj2 in layer.getFeatures():
            if obj1.id() != obj2.id() and not obj1 in obiektyZbledami and obj1.geometry().distance(obj2.geometry()) < 2 :
                obiektyZbledami.append(obj1)
                break
    return obiektyZbledami


def czyOdleglosciMiedzyPoziomicami2m(layer):
    obiekty_z_bledami = []
    spatial_index = QgsSpatialIndex(layer.getFeatures(), flags=QgsSpatialIndex.FlagStoreFeatureGeometries)
    for obj1 in layer.getFeatures():
        nearestNeighbors = spatial_index.nearestNeighbor(obj1.geometry(), 2, 0)
        for nn in nearestNeighbors:
            if obj1.id() != nn:
                for obj2 in layer.getFeatures():
                    if nn == obj2.id() and not obj1 in obiekty_z_bledami and obj1.geometry().distance(obj2.geometry()) < 2 and \
                        obj1.geometry().distance(obj2.geometry()) > 0:
                        obiekty_z_bledami.append(obj1)
                        break
    return obiekty_z_bledami


def nadmiernaSegmentacja(layer):
    id_to_feature = {feature.id(): feature for feature in layer.getFeatures()} # Tworzenie słownika mapującego ID obiektu na obiekt
    obiekty_z_bledami = []
    spatial_index = QgsSpatialIndex(layer.getFeatures(), flags=QgsSpatialIndex.FlagStoreFeatureGeometries)
    for obj1 in layer.getFeatures():
        nearestNeighbors = spatial_index.nearestNeighbor(obj1.geometry(), 2, 0)
        for nn in nearestNeighbors:
            if obj1.id() == nn:
                continue
            obj2 = id_to_feature[nn]
            atrybuty1 = obj1.attributes()[2:]
            atrybuty2 = obj2.attributes()[2:]
            if not obj1 in obiekty_z_bledami and obj1.geometry().distance(obj2.geometry()) < 0.01 and atrybuty1 == atrybuty2 and not obj1.geometry().equals(obj2.geometry()):
                obiekty_z_bledami.append(obj1)
                break
    return obiekty_z_bledami


def przewerteksowanie(layer):
    obiekty_z_bledami = []
    for obj1 in layer.getFeatures():
        geom = obj1.geometry()
        liczbaWerteksów = len(geom.asPolyline())
        if liczbaWerteksów > 2:
            for i in range(0, liczbaWerteksów):
                if i > 0 and i < liczbaWerteksów:
                    odcinek_linii = QgsLineString()
                    odcinek_linii.addVertex(geom.vertexAt(i-1))
                    odcinek_linii.addVertex(geom.vertexAt(i+1))
                    geom1 = QgsGeometry(odcinek_linii)
                    d = geom1.distance(QgsGeometry(geom.vertexAt(i)))
                    if d < 0.05:
                        obiekty_z_bledami.append(obj1)
                        return obiekty_z_bledami
    return obiekty_z_bledami


def kontrola_OT_ADMS_P_z_OT_ADMS_A(layer): #topo_e3_k5
    obiekty_z_bledami = []
    OT_ADMS_A_layer = None
    OT_ADMS_P_layer = None
    if layer.name()[-6:] == 'ADMS_P':
        OT_ADMS_A_layer = QgsProject().instance().mapLayersByName(layer.name().replace("OT_ADMS_P","OT_ADMS_A"))[0]  
        OT_ADMS_P_layer = QgsProject().instance().mapLayersByName(layer.name())[0]  
    if OT_ADMS_A_layer and OT_ADMS_P_layer:
        identyfikatory_bledow = set()  # Zbiór do śledzenia identyfikatorów błędnych obiektów
        nazwy_adms_p = set([feature['nazwa'] for feature in OT_ADMS_P_layer.getFeatures()])
        nazwy_adms_a = set([feature['nazwa'] for feature in OT_ADMS_A_layer.getFeatures()])
        # Sprawdzenie przecięcia i nazw dla obiektów z OT_ADMS_A
        for OT_ADMS_A_feature in OT_ADMS_A_layer.getFeatures():
            is_intersected = False
            nazwa_adms_a = OT_ADMS_A_feature['nazwa']
            # Sprawdzenie przecięcia dla każdego obiektu z OT_ADMS_P
            for OT_ADMS_P_feature in OT_ADMS_P_layer.getFeatures():
                nazwa_adms_p = OT_ADMS_P_feature['nazwa']
                if OT_ADMS_P_feature.geometry().intersects(OT_ADMS_A_feature.geometry()) or OT_ADMS_A_feature.geometry().contains(OT_ADMS_P_feature.geometry()):
                    is_intersected = True
                    break
            if not is_intersected or nazwa_adms_a not in nazwy_adms_p: #.. oraz czy nazwy z OT_ADMS_A występują w OT_ADMS_P
                if OT_ADMS_A_feature.id() not in identyfikatory_bledow:
                    obiekty_z_bledami.append(OT_ADMS_A_feature)
                    identyfikatory_bledow.add(OT_ADMS_A_feature.id())
                
            # Sprawdzenie, czy nazwy z OT_ADMS_P występują w OT_ADMS_A
            if OT_ADMS_P_feature['nazwa'] not in nazwy_adms_a and OT_ADMS_P_feature.id() not in identyfikatory_bledow:
                obiekty_z_bledami.append(OT_ADMS_P_feature)
                identyfikatory_bledow.add(OT_ADMS_P_feature.id())      
                
        return obiekty_z_bledami


def kontrolaTERCpunkt(layer):
    obiektyZbledami = []
    adms = None
    adja = None
    if layer.name()[-6:] == 'ADMS_P':
        adja = QgsProject().instance().mapLayersByName(layer.name().replace("OT_ADMS_P","OT_ADJA_A"))[0]  
        adms = QgsProject().instance().mapLayersByName(layer.name())[0] 
        
    # Zestaw do śledzenia już dodanych identyfikatorów TERYT
    przetworzone_teryt = set()
    
    # Sprawdzenie, czy są warstwy
    if adms and adja:# and all(z not in layer for z in dalszeWarstwy):
        expression = """ "rodzaj" IN ('miasto', 'miasto w gminie miejsko-wiejskiej', 'gmina') """
        # Wykonaj selekcję
        adja.selectByExpression(expression, QgsVectorLayer.SetSelection)
        selected = adja.selectedFeatures()
        # Przygotuj listę identyfikatorów TERYT z warstwy ADJA jako tekst
        identyfikatory_teryt = {str(feature["identyfikatorTerytJednostki"]).zfill(7): feature for feature in selected}
        # Przygotuj słownik dla ADMS_P: klucz to IdentyfikatorTERC, wartość to lista nazw
        identyfikatory_terc_adms = {}
        for feature in adms.getFeatures():
           teryt_adms = str(feature["IdentyfikatorTERC"]).zfill(7)
           if teryt_adms in identyfikatory_terc_adms:
               identyfikatory_terc_adms[teryt_adms].append(feature)
           else:
               identyfikatory_terc_adms[teryt_adms] = [feature]  
               
        # Dla każdego identyfikatora TERYT/TERC
        for teryt, adja_feature in identyfikatory_teryt.items():
            if not teryt in identyfikatory_terc_adms and len(teryt) == 7:
                obiektyZbledami.append(adja_feature)
                
        for teryt_adms, adms_features in identyfikatory_terc_adms.items():
           if teryt_adms not in identyfikatory_teryt:
               for adms_feature in adms_features:
                   obiektyZbledami.append(adms_feature)  
                           
        adja.removeSelection()
    return obiektyZbledami


def kontrolaTERCpowierzchnia(layer):
    obiektyZbledami = []
    adms = None
    adja = None
    if layer.name()[-6:] == 'ADMS_A':
        adja = QgsProject().instance().mapLayersByName(layer.name().replace("ADMS","ADJA"))[0]  
        adms = QgsProject().instance().mapLayersByName(layer.name())[0] 
    
    # Zestaw do śledzenia już dodanych identyfikatorów TERYT
    przetworzone_teryt = set()
    indeks_przestrzenny = QgsSpatialIndex()  # Tworzenie nowego indeksu przestrzennego
    # Sprawdzenie, czy jsą warstwy
    if adms and adja:
        # Iteruj przez warstwy
        expression = """ "rodzaj" IN ('miasto', 'miasto w gminie miejsko-wiejskiej', 'gmina') """
        # Wykonaj selekcję na ADJA_A
        adja.selectByExpression(expression, QgsVectorLayer.SetSelection)
        selected = adja.selectedFeatures()
        # Przygotuj listę identyfikatorów TERYT z warstwy ADJA jako tekst
        identyfikatory_teryt = {str(feature["identyfikatorTerytJednostki"]).zfill(7): feature  for feature in selected}
        # Przygotuj słownik dla ADMS_A: klucz to IdentyfikatorTERC, wartość to lista nazw
        identyfikatory_terc_adms = {}
        for feature in adms.getFeatures():
            teryt_adms = str(feature["IdentyfikatorTERC"]).zfill(7)
            if teryt_adms in identyfikatory_terc_adms:
                identyfikatory_terc_adms[teryt_adms].append(feature)
            else:
                identyfikatory_terc_adms[teryt_adms] = [feature]
        
        # Dodawanie obiektów z ADMS i ADJA do indeksu przestrzennego
        for feature in adms.getFeatures():
            indeks_przestrzenny.addFeature(feature)
        for feature in selected:  # Uwaga: 'selected' zawiera już tylko wyselekcjonowane obiekty z 'adja'
            indeks_przestrzenny.addFeature(feature)
            
        # Sprawdź pokrycie
        for teryt, adja_feature in identyfikatory_teryt.items():
            if not teryt in identyfikatory_terc_adms and len(teryt) == 7:
                obiektyZbledami.append(adja_feature)
                
         # Dodatkowe sprawdzenie: kod TERYT w ADMS_A, ale nie w ADJA
        for teryt_adms, adms_features in identyfikatory_terc_adms.items():
           if teryt_adms not in identyfikatory_teryt:
               for adms_feature in adms_features:
                    obiektyZbledami.append(adms_feature) 
                    
        adja.removeSelection()
    return obiektyZbledami


def fullCoverage(layer):
    # Tworzenie listy obiektów warstw
    adja = None
    pokrycie = []
    root = QgsProject.instance().layerTreeRoot()
    for group in root.children():
        if isinstance(group, QgsLayerTreeGroup):
            # Iteracja przez wszystkie warstwy w grupie
            layerIds = group.findLayerIds()
            if layerIds:  # Sprawdzenie, czy layerIds nie jest None
                for layerId in layerIds:
                    la = QgsProject.instance().mapLayer(layerId)
                    if isinstance(la, QgsVectorLayer) and 'PT' in layerId:
                    # Dodanie warstwy wektorowej do listy 'pokrycie'
                        pokrycie.append(la)
                    if isinstance(la, QgsVectorLayer) and 'ADJA_A' in layerId:
                        adja = la             
    # dodanie warstwy, gdy ma wszystkie przecięcia   
    przetworzone_warstwy = set()
    # Tworzenie indeksu przestrzennego dla każdej warstwy wektorowej
    obiekty_wszystkie = []
    for l in pokrycie:
        for feature in l.getFeatures():
            obiekty_wszystkie.append((feature, l))
    # Tworzenie wspólnego indeksu przestrzennego
    indeks_przestrzenny = QgsSpatialIndex()
    python_version = sys.version_info.major
    if python_version < 3:
        for idx, (feature, _) in enumerate(obiekty_wszystkie):
            indeks_przestrzenny.insertFeature(feature)
    else:    
        for feature, _ in obiekty_wszystkie:
            indeks_przestrzenny.addFeature(feature)
    for feature, _ in obiekty_wszystkie:
        indeks_przestrzenny.addFeature(feature)
    # Iteracja przez każdą parę warstw do sprawdzenia pokryć
    if adja and len(pokrycie) == 12 and not QgsProject.instance().mapLayersByName("Nakładanie") and not QgsProject.instance().mapLayersByName("Dziury"):
        nakladanie = QgsVectorLayer("Polygon?crs=epsg:2180&field=lokalnyId:string(254)", "Nakładanie", "memory")
        pr = nakladanie.dataProvider()
        for i, (feature_i, layer_i) in enumerate(obiekty_wszystkie):  # Pomijamy ostatni obiekt, ponieważ nie ma z czym przecinać
        # Pobieranie ID obiektów, które mogą się przecinać
            ids_potencjalnie_przecinajace = indeks_przestrzenny.intersects(feature_i.geometry().boundingBox())
            for id_potencjalne in ids_potencjalnie_przecinajace:
                if id_potencjalne != i:  # Unikamy porównywania obiektu z samym sobą
                    feature_j, layer_j = obiekty_wszystkie[id_potencjalne]
                    if feature_i.geometry().intersects(feature_j.geometry()):
                        # Dodajemy przecinające się obiekty do nowej warstwy 'nakladanie'
                        nowy_feature = QgsFeature(pr.fields())
                        nowy_feature.setGeometry(feature_i.geometry().intersection(feature_j.geometry()))
                        nowy_feature['lokalnyId'] = feature_j['lokalnyId'] 
                        pr.addFeature(nowy_feature)
                        nakladanie.updateExtents()
        QgsProject.instance().addMapLayer(nakladanie)
        # obliczanie przecięć /\  i dziur- braków między warstwami \/
        invalid_layer = processing.run("qgis:checkvalidity", {'INPUT_LAYER': adja, 'METHOD': 0, 'IGNORE_RING_SELF_INTERSECTION': True, 'VALID_OUTPUT': 'memory:', 'INVALID_OUTPUT': 'memory:', 'ERROR_OUTPUT': 'memory:'})
            # Krok 3: obliczanie dziur (braków w pokryciu) między warstwami
            # zawiera warstwę z niepoprawnymi geometriami
        if invalid_layer['INVALID_OUTPUT'] and not QgsProject.instance().mapLayersByName("naprawiona adja"):
                  params = {
                  'INPUT': adja,
                  'OUTPUT': 'memory:'
                  }
              # Wykonanie naprawy geometrii
                  naprawiona= processing.run("native:fixgeometries", params)['OUTPUT']
                  naprawiona.setName("naprawiona adja")
                  adja = naprawiona        
        if not QgsProject.instance().mapLayersByName("Dziury"): # warstwa ma się utworzyć tylko raz
            dziury = []
            for i in pokrycie: 
                params_difference = {
                      'INPUT': i,
                      'OVERLAY': adja,
                      'OUTPUT': 'memory:'
                }
                dissolve = processing.run('native:dissolve', params_difference)['OUTPUT']
                dissolve.setName(f"Dziury i.name()")
                dziury.append(dissolve)
                alg_params = { # obliczenie różnicy
                    'INPUT': adja, # na początek idzie warstwa ADJA
                    'OVERLAY': dissolve,
                    'OUTPUT': 'memory:'
                    }
                adja = processing.run('native:difference', alg_params)['OUTPUT'] # do zmiennej trafiają z kolejną iteracją wyniki z poprzedniej iteracji.
                adja.setName("Dziury")
                QgsProject.instance().removeMapLayer(dissolve.id())
                # Dodaj połączoną warstwę do projektu
            QgsProject.instance().addMapLayer(adja)


def boundaryPTWP(layer):
    obiektyZbledami = []
    ptwp = None
    rtlw = None
    if layer.name()[-6:] == 'RTLW_L':
        ptwp = QgsProject().instance().mapLayersByName(layer.name().replace("RTLW_L","PTWP_A"))[0]  
        rtlw = QgsProject().instance().mapLayersByName(layer.name())[0] 
    if ptwp and rtlw:
        # Iteracja przez każdy obiekt w warstwie RTLW  
        for rtlw_feature in rtlw.getFeatures():
            rtlw_geom = rtlw_feature.geometry()
            rodzaj_rtlw = rtlw_feature['rodzaj']
            # Iteracja przez każdy obiekt w warstwie PTWP
            for ptwp_feature in ptwp.getFeatures():
                ptwp_geom = ptwp_feature.geometry() 
            # Sprawdzanie przecięcia wnętrza (ignoruje stykanie się)
                if (rtlw_geom.crosses(ptwp_geom) or (rtlw_geom.intersects(ptwp_geom)) and not rtlw_geom.touches(ptwp_geom)) and rodzaj_rtlw in ('skarpa','wąwóz'):
                    obiektyZbledami.append(rtlw_feature) 
                    
    return obiektyZbledami               



def gminyCzyNakladajaSie(layer):
    obiektyZbledami = []     
    gm = processing.run("qgis:extractbyexpression", {
        'INPUT':layer,   
        'EXPRESSION':'"rodzaj" = \'gmina\'',
        'OUTPUT':'memory:'})
    gminy = gm['OUTPUT']
    gminy.setName("gminy")
    
    invalid_layer = processing.run("qgis:checkvalidity", {
        'INPUT_LAYER': gminy, 
        'METHOD': 0, 
        'IGNORE_RING_SELF_INTERSECTION': True,
        'VALID_OUTPUT': 'memory:', 
        'INVALID_OUTPUT': 'memory:', 
        'ERROR_OUTPUT': 'memory:'})
        # zawiera warstwę z niepoprawnymi geometriami
    if invalid_layer['INVALID_OUTPUT']:
              params = {
              'INPUT': gminy,
              'OUTPUT': 'memory:'
              }
          # Wykonanie naprawy geometrii
              naprawiona= processing.run("native:fixgeometries", params)['OUTPUT']
              naprawiona.setName("naprawiona ADJA_A")
              gminy = naprawiona        
    QgsProject.instance().addMapLayer(gminy) 
    
    nach = processing.run("qgis:extractbylocation", {
        'INPUT': gminy,
        'PREDICATE': [5],
        'INTERSECT': gminy,
        'OUTPUT': 'memory:'})
    wynik = nach['OUTPUT']
    wynik.setName("wynik") 
    QgsProject.instance().addMapLayer(wynik)
    for obj in wynik.getFeatures():
        obiektyZbledami.append(obj)
    return obiektyZbledami


def miastoWiesCzyNakladajaSie(layer):
    obiektyZbledami = []
    mw = processing.run("qgis:extractbyexpression",{
        'INPUT': layer,
        'EXPRESSION':'"rodzaj"=\'miasto\' or "rodzaj"=\'wieś\'',
        'OUTPUT':'memory:'})
    miastoWies = mw['OUTPUT']
    miastoWies.setName("miasto_wies")
    
    invalid_layer = processing.run("qgis:checkvalidity",{
        'INPUT_LAYER': miastoWies,
        'METHOD': 0,
        'IGNORE_RING_SELF_INTERSECTION': True,
        'VALID_OUTPUT': 'memory:',
        'INVALID_OUTPUT': 'memory:',
        'ERROR_OUTPUT': 'memory:'})

        # zawiera warstwę z niepoprawnymi geometriami  
    if invalid_layer['INVALID_OUTPUT']:
              params = {
              'INPUT': miastoWies,
              'OUTPUT': 'memory:'
              }  
          # Wykonanie naprawy geometrii
              naprawiona= processing.run("native:fixgeometries", params)['OUTPUT']
              naprawiona.setName("naprawiona ADMS_A")
              miastoWies = naprawiona                
    QgsProject.instance().addMapLayer(miastoWies) 
    
    nach = processing.run("qgis:extractbylocation", {
        'INPUT': miastoWies,
        'PREDICATE': [5],
        'INTERSECT': miastoWies,
        'OUTPUT': 'memory:'})
    wynik = nach['OUTPUT']
    wynik.setName("wynik")
    for obj in wynik.getFeatures():        
        obiektyZbledami.append(obj)
    return obiektyZbledami


def OT_SKDR_L_z_OT_PTTR_A(layer):

    obiektyZbledami = []  
    OT_PTTR_A = None
    OT_SKDR_L = None
    # if layer.name()[-6:] == 'PTTR_A':
    #     OT_SKDR_L = QgsProject().instance().mapLayersByName(layer.name().replace("PTTR_A","SKDR_L"))[0]  
    #     OT_PTTR_A = QgsProject().instance().mapLayersByName(layer.name())[0] 
    root = QgsProject.instance().layerTreeRoot()
    for child in root.children():
        if isinstance(child, QgsLayerTreeGroup):
            layerIds = child.findLayerIds()
            for layerId in child.findLayerIds():
                la = QgsProject.instance().mapLayer(layerId)
                if isinstance(la, QgsVectorLayer) and 'OT_PTTR_A' in layerId:
                    OT_PTTR_A_layer = la
                if isinstance(la, QgsVectorLayer) and 'OT_SKDR_L' in layerId:
                    OT_SKDR_L_layer = la
    if OT_PTTR_A_layer and OT_SKDR_L_layer:        
        
        # wybor roslinnosci trawiastej
        traw = processing.run("qgis:extractbyexpression", {
            'INPUT':OT_PTTR_A_layer,
            'EXPRESSION':'"rodzaj" = \'roślinność trawiasta\'',
            'OUTPUT':'memory:'})
        rosTraw = traw['OUTPUT']
        rosTraw.setName("roślinność trawiasta")
        QgsProject.instance().addMapLayer(rosTraw)
        print(rosTraw) 
        
        # obliczenie odleglosci miedzy jezdniami
        szer_1_2 = processing.run("qgis:fieldcalculator", {
            'INPUT':OT_SKDR_L_layer,
            'FIELD_NAME':'1_2',
            'FORMULA':' "szerokoscNawierzchni" / 2',
            'OUTPUT':'memory:'})
        odlJezdni = szer_1_2['OUTPUT']
        odlJezdni.setName("odległość między jezdniami - szerokość pasa zieleni")
        QgsProject.instance().addMapLayer(odlJezdni)
        print(odlJezdni) 
    
        # odleglosc miedzy jezdniami > 5m
        szer_1_2_5 = processing.run("qgis:extractbyexpression", {
            'INPUT':odlJezdni,
            'EXPRESSION':'"1_2" > 5',
            'OUTPUT':'memory:'})
        odlJezdPowyzej5 = szer_1_2_5['OUTPUT']
        odlJezdPowyzej5.setName("odległość między jezdniami > 5m")
        QgsProject.instance().addMapLayer(odlJezdPowyzej5)
        print(odlJezdPowyzej5)

        buf = processing.run("qgis:buffer", {
            'INPUT':odlJezdPowyzej5,
            'DISTANCE':QgsProperty.fromExpression('coalesce(scale_linear("szer_1_2")'),
            'SEGMENTS':5,
            'END_CAP_STYLE':0,
            'JOIN_STYLE':0,
            'MITER_LIMIT':2,
            'DISSOLVE':False,
            'SEPARATE_DISJOINT':False,
            'OUTPUT':'memory:'})
        bufor = buf['OUTPUT']
        bufor.setName("bufor od pasa zieleni szerszego niż 5m")
        QgsProject.instance().addMapLayer(bufor)
        print(bufor)
        
        przec = processing.run("qgis:intersection", {
            'INPUT':rosTraw,
            'OVERLAY':bufor,
            'OUTPUT':'memory:'})
        iloczyn = przec['OUTPUT']
        iloczyn.setName("pas zieleni szerszy niż 5m")
        QgsProject.instance().addMapLayer(iloczyn)
        print(iloczyn)
        
        powPZ = processing.run("qgis:fieldcalculator", {
            'INPUT':iloczyn,
            'FIELD_NAME':'pow',
            'FIELD_TYPE':0,
            'FIELD_LENGTH':0,
            'FIELD_PRECISION':0,
            'FORMULA':'$area',
            'OUTPUT':'memory:'})
        powPasZiel = powPZ['OUTPUT']
        powPasZiel.setName("powierzchnia pasa zieleni")
        QgsProject.instance().addMapLayer(powPasZiel)
        print(powPasZiel)
                
        powPZ_500_1000 = processing.run("qgis:extractbyexpression", {
            'INPUT':powPasZiel,
            'EXPRESSION':'"pow" < 500 or "pow" > 1000',
            'OUTPUT':'memory:'})  
        wynik = powPZ_500_1000['OUTPUT']
        wynik.setName("powierzchnia pasa zieleni < 500 lub > 1000")
        QgsProject.instance().addMapLayer(wynik)
        print(wynik)
        
        for obj in wynik.getFeatures():        
            obiektyZbledami.append(obj)
        return obiektyZbledami
    